
/***             Program developed for LogicFlex--EPX                 ***/

/************************************************************************/
/*								                                             	*/
/*	Copyright JK microsystems 2004	-  All Rights Reserved	         	*/
/*								                                             	*/
/*	This software is NOT shareware, freeware, or public domain.       	*/
/*	It is the property of JK microsystems.			                       	*/
/*							                                              		*/
/*	Customers of JK microsystems may modify the source code	         	*/
/*	and/or distribute the binary image of this software without       	*/
/*	additional costs provided it is run only on hardware           		*/
/*	manufactured by JK microsystems.  All other use is expressly      	*/
/*	prohibited.						                                       	*/
/*							                                                  	*/
/*	Update Log						                                        	*/
/*								                                              	*/
/*	Version	 Date	    Comments			 Progammer                      	*/
/*	-------	------	 ------------------------------- ----------	      */
/*	1.0	   070804	 First release	 		 bjp		                     */
/*								                                              	*/
/*							                                               		*/
/************************************************************************/
/*
	This is a complete software package for the LogicFlex-EPX.
	This package drives the internal 2X16 LCD, monitors user-pressed buttons,
	controls the I/O ports, establishes remote control ability between multiple
	LogicFlex-EXP devices using TCP/IP, allows users a telnet server interface
	for remote monitoring, and provides a complete user interface.  The compiled
	executable file has been preinstalled in your LogicFlex-EPX.

	The structure of this program has been built using a State-Machine based
	implementation.  Control variables, flags, and 3 levels of data structures
	are active throughout the program. This provides a variety of user options
	including: TCP/IP connectivity, memory capability, and enhanced performance.
*/

#include <stdio.h>
#include <time.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <bios.h>
#include <dos.h>
#include <time.h>
#include <ctype.h>   /* For isdigit() */

/* Lets compiler know that this header should be treated as a C file */
extern "C"
{
#include "epx_drvr.h"
/* wattcp is a C library, we're using the C++ compiler */
#include <tcp.h>
}

#define TELNET_PORT 23  	/* Standard telnet interface port */
#define LCD_CMD 160			/* Command for LCD driver */
#define LCD_CMD1 40    	  	/* I/O format setup command */
#define LCD_CMD2 6     		/* Cursor setup command */
#define LCD_CMD3 6      	/* Cursor setup command */
#define CLR_HOME 1    	   /* Clear and home command */

#define Button_Mem_Location 0x800   /* Inportb(0x800);  reading from
													this location returns a 8-bit status */
#define MINUS_Mask 0x20    /* Minus button */
#define PLUS_Mask  0x10    /* Plus button */
#define RIGHT_Mask 0x08    /* Right button */
#define LEFT_Mask  0x04    /* Left button */
#define DOWN_Mask  0x02    /* Down button */
#define UP_Mask    0x01    /* Up button */

extern char *_sys_errlist[];

FILE *lcd;                          /* Stream for LCD data */
unsigned char key_status = 0;
int Max_Limit[3] = { 7, 1, 3 };
int arrow_position[3] = { 0, 0, 0 };
int LCD_current_line1[3] = { 0, 0, 0 };
int LCD_current_line2[3] = { 1, 1, 1 };
int level = 0;
int MSmode = 0;                     /* Client by default*/
char select_string[12]   = { "0123456789.\0" };
char ASCII_string[50]   = { "abcdefghijklmnopqrstuvwxyz 1234567890.?!@#$%^&*()\0" };
char flash_string1[17]  = { "_              +\0" };
char flash_string2[17]  = { "_              +\0" };
char flash_string3[17]  = { "_              +\0" };
char flash_string4[17]  = { "_              +\0" };
char flash_string5[17]  = { "_               \0" };
char flash_string6[17]  = { " 000000000000000\0" };
char flash_string7[17]  = { "TIME   _:__:__ +\0" };
char own_IP[17]         = { "               +\0" };
char remote_IP[17]      = { "               +\0" };
char message[17]        = { "                \0" };
char temp_string[17]    = { "                \0" };
char set_port_string[17]     = { "               +\0" };
char clear_port_string[17]   = { "               +\0" };
char server_output_string[17]= { "0000000000000000\0" };
char CS[7] = {"      "};
char relay_in_string[47]  = {"            Inputs:        0000000000000000   "};
char relay_out_string[47] = {"            Output Relays: 0000000000000000   "};
char mode_string[47]      = {"            This LogicFlex EPX is a:          "};
char message_string[47]   = {"            Incoming Message:                 "};
char date_time_string[47] = {"                                              "};
char output[17]    = { "Outgoing Message:" };
int continue_flag  = 1;
int continue_flag2 = 1;
int continue_flag3 = 1;
int pull_time_flag = 1;

char mytime[17] = { "TIME  __:__:__ +\n" };
char mydate[17] = { "DATE  __-__-____\n" };
unsigned char flash_cursor_pos = 0x80;

/*
** Define user-defined character dot patterns
*/
const unsigned char LCD_CHAR_PLUS[] = {
  0x00,   // ---00000
  0x04,   // ---00100
  0x06,   // ---00110
  0x1f,   // ---11111
  0x16,   // ---10110
  0x14,   // ---10100
  0x00,   // ---00000
  0x00,   // ---00000
};

const unsigned char LCD_CHAR_MINUS[] = {
  0x00,   // ---00000
  0x05,   // ---00101
  0x0d,   // ---01101
  0x1f,   // ---11111
  0x0c,   // ---01100
  0x04,   // ---00100
  0x00,   // ---00000
  0x00,   // ---00000
};

int pull_message_continue = 1;
int exe_this = 0;
char Level2Menu[4][17] = {
									{ "->Message Server\0" },
									{ "  Monitor Input \0" },
									{ "  Control Output\0" },
									{ "  Disconnect... \0" }
								 };

char buffer[17];						/* Buffer for use in telnets() for Enter button */
char rbuffer[17];						/* Buffer for Rx data*/
char tbuffer[17];						/* Buffer for Tx data*/
char temp_rbuffer[17];           /* Buffer for new inputs for server */
char temp_tbuffer[17];           /* Buffer for old inputs for server */
char telnet_buffer[17];          /* Buffer for old inputs for telnets */
char telnet_buffer1[17];         /* Buffer for new inputs for */
char temp_lcd[17];               /* Buffer for Tx data*/

int status1;                     /* Status of socket 1 */
int status;                      /* Status of socket 2 */
int m_establish = 0;
int s_establish = 0;
int x = 0;

const static max_sockets = 3;  				/* Maximum connections to the server */
static tcp_Socket s[max_sockets];			/* Declare a socket structure for each server connection */

int periodic_TCP = 0;
time_t sec_now;
time_t currenttime1;
time_t elapsetime = NULL;
int AP_24 = 0;
int port;
char t[17] = { "<-              \0" };
char d[17] = { "                \0" };

typedef struct Submenu
{
	char SubOption[2][17];
};

typedef struct Menu
{
	char Option[17];
	struct Submenu* sub;
};

struct Submenu S[7] =
{
	{ {"-> Set to Client\0", "   Set to Server\0"} },
	{ {"-> Set your IP: \0", "   Set remote IP\0"} },
	{ {" Connecting...  \0", "EPX~~~       EPX\0"} },
	{ {"-> Set Relay    \0", "   Clear Relay  \0"} },
	{ {"0123456789ABCDEF\0", "                \0"} },
	{ {"-> Set Time,Date\0", "   AM/PM? 24 hr?\0"} },
	{ {"                \0", "                \0"} }
};

struct Menu M[8] =
{
	{ "->Client/Server?\0", &S[0] },
	{ "  Set IP Address\0", &S[1] },
	{ "  Connect to WWW\0", &S[2] },
	{ "  Output Relay  \0", &S[3] },
	{ "  Input  Relay  \0", &S[4] },
	{ "  Time Setting  \0", &S[5] },
	{ "  Clock         \0", &S[6] },
	{ "  Return to OS  \0", &S[7] }
};

void DispClock( void );
void InputRelays( void );
void Intro( void );
void Mmenu(int &level, int arrow_position[], int Max_Limit[]);
int SubMenSel (int level, int arrow_position[], int Max_Limit[], int up);
int SubMen12 (int level, int arrow_position[], int up);
void ClintServ (int arrow_position[]);
void SetIP (int arrow_position[]);
int ConnWWW ( void );
void RelayCont ( void );
void TimeSet ( void );
void TimeSet_DigitsUD(int h, int i);
void TimeSet_DigitsLRa(int h);
void TimeSet_DigitsLRb(int h);
int RetOS ( int quit );
void WWWConnCont ( void );
void DebounceUP(void);
void DebounceDOWN(void);
void DebounceLEFT(void);
void DebounceRIGHT(void);

/************   Functions   ******************/
/******************************************************************************/
void LCD_DefineChar (char address, const unsigned char *pattern)
/*
** LCD_DefineCharacter: Define dot pattern for user-defined characters
** and puts them in the LCD Character Generator RAM memoery.
** inputs: address = address of character (0x00-0x07)
**         pattern = pointer to 8-byte array containing the dot pattern
*/
{
  char i;

  fprintf(lcd,"%c%c",LCD_CMD,(0x40 + (address << 3)));
  for (i=0; i<8; i++)
  {
		fprintf(lcd,"%c",pattern[i]);
  }
}
/******************************************************************************/

/******************************************************************************/
void LCDRefresh()
{
	/* Need to scroll ? */
	if( arrow_position[level] > LCD_current_line2[level] )
	{
		/* Scroll down */
		LCD_current_line1[level]++;
		LCD_current_line2[level]++;
	}
	else if( arrow_position[level] < LCD_current_line1[level] )
	{
		/* Scroll up */
		LCD_current_line1[level]--;
		LCD_current_line2[level]--;
	}
	if( level == 0 )
	{
		/* Writes info to the display */
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, M[ LCD_current_line1[level] ].Option );
		fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
		fprintf( lcd, M[ LCD_current_line2[level] ].Option );
	}
	else if( level == 1 && arrow_position[0] < 7)
	{
		/* Writes info to the display*/
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, (M[ arrow_position[0] ].sub)->SubOption[ LCD_current_line1[1] ] );
		fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
		fprintf( lcd, (M[ arrow_position[0] ].sub)->SubOption[ LCD_current_line2[1] ] );

		if( arrow_position[0] == 2 )
			exe_this = 1;
		else if( arrow_position[0] == 4 )
		{
		  InputRelays ();
		}
		else if( arrow_position[0] == 6 )
		{
         if ( sock_established(&s[1]))
			{
				status1 = 0;
				Mmenu(level, arrow_position, Max_Limit);
			}
			DispClock ();
		}
	}
	else if( level == 2 )
	{
		/* Only Client comes in here... */
		/* Display... */
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, Level2Menu[ LCD_current_line1[level] ] );
		fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
		fprintf( lcd, Level2Menu[ LCD_current_line2[level] ] );
	}
	time(&currenttime1);
	elapsetime = NULL;
}
/*****************************************************************************/

/*****************************************************************************/
void telnets( int port )
{
	int i = 0;

	if ( !(sock_established( &s[1] )))
	{
		tcp_listen( &s[1], port, 0L, 0, NULL, 0);

		tcp_tick(&s[1]);                                    /* Must use multiple ticks to have time to connect. */
		tcp_tick(&s[1]);
	}
	if ( sock_established(&s[1]))
	{
		sock_mode( &s[1], TCP_MODE_ASCII );
		puts("Receiving incomming connection");
		key_status = inportb(Button_Mem_Location);
		if ( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed*/
			return;

		if (MSmode == 0)
			strcpy( CS, "Client\0");
		else
			strcpy( CS, "Server\0");
		for ( i = 0; i < 16; i++ )     /* Gets relay inputs and outputs and fills the corresponding strings. */
		{
			if (MSmode == 0)            /* Client mode */
			{
				message_string[i+12] = output[i];        /* Changes Incoming to Outgoing in message_string */
				if ( strncmp(tbuffer, "Monitor", 7 ) == 0 )  /* For Monitor Inputs only */
				{
					message_string[i+30] = rbuffer[i];
				}
				else
				{
					message_string[i+30] = tbuffer[i];        /* Fills message string*/
				}
				if (set_port_string[i] == '1' || set_port_string[i] == '0' )
					relay_out_string[i+27] = set_port_string[i];   /* Fills Output Relays: */
				if (GetIOPt(i)+48 == '1')
				{
					relay_in_string[i+27] = (GetIOPt(i)+48);       /* Fills Input Relays for Client */
					telnet_buffer[i] = relay_in_string[i+27];
				}
				else if (x == 1)
				{
					relay_in_string[i+27] = telnet_buffer1[i];       /* Fills Input Relays for Client if changes happen while in telnets */
					x = 0;
				}
			}
			if (MSmode == 1)    /* Server mode -- Refreshes if there is new data */
			{
				message_string[i+30] = temp_rbuffer[i];                /* Fills message string*/
				if ( set_port_string[i] == '1' || set_port_string[i] == '0' )
					relay_out_string[i+27] = set_port_string[i];   /* Fills Output Relays: */
				if (x == 1)
				{
					relay_in_string[i+27] = telnet_buffer1[i];       /* Fills Input Relays for Client if changes happen while in telnets */
					x = 0;
				}
				else
				{
					relay_in_string[i+27] = temp_tbuffer[i];       /* Fills Input Relays for Server */
					telnet_buffer[i] = temp_tbuffer[i];
				}
			}
		}
		telnet_buffer[17]= '\0';
		for ( i = 0; i < 6; i++)                      /* Fills mode string. */
		{
			mode_string[i+37] = CS[i];
		}
		time(&sec_now);  /* Get current time (in seconds)*/
		strncpy( &t[3], ctime(&sec_now)+11, 8 );     /* Gets current time. */
		strncpy( &d[3], ctime(&sec_now), 10);        /* Gets current date. */
		for ( i = 0; i < 16; i++)  					   /* Fills string with current time and date. */
		{
			date_time_string[i+12] = t[i+2];
			date_time_string[i+28] = d[i];
		}
		date_time_string[26] = ' ';

		sock_puts( &s[1], (unsigned char *)"\33c");     /* Reset screen to initial state. */
		sock_puts( &s[1], (unsigned char *)"      Welcome to the LogicFlex-EPX telnet server.\n");
		sock_puts( &s[1], (unsigned char *)"          This table lists information about  ");
		sock_puts( &s[1], (unsigned char *)"              your LogiFlex-EPX system.       \n");
		sock_puts( &s[1], (unsigned char *)"          ------------------------------------");
		sock_puts( &s[1], (unsigned char *) mode_string );
		sock_puts( &s[1], (unsigned char *)"          ------------------------------------");
		sock_puts( &s[1], (unsigned char *)"            Relay #        0123456789ABCDEF   ");
		sock_puts( &s[1], (unsigned char *)"          ------------------------------------");
		sock_puts( &s[1], (unsigned char *) relay_in_string );
		sock_puts( &s[1], (unsigned char *)"          ------------------------------------");
		sock_puts( &s[1], (unsigned char *) relay_out_string );
		sock_puts( &s[1], (unsigned char *)"          ------------------------------------");
		sock_puts( &s[1], (unsigned char *) message_string );
		sock_puts( &s[1], (unsigned char *)"          ------------------------------------");
		sock_puts( &s[1], (unsigned char *) date_time_string );
		sock_puts( &s[1], (unsigned char *)"          ------------------------------------\n");
		sock_puts( &s[1], (unsigned char *)"       The screen will refresh if any of the data");
		sock_puts( &s[1], (unsigned char *)"                  changes or you may            ");
		sock_puts( &s[1], (unsigned char *)"       press the Enter key to refresh the screen.");
		do
		{
			if (!tcp_tick( &s[1] ))
			{
				status1 = 1;
				break;
			}
			for(int i=0; i<16; i++)
			{
				telnet_buffer1[i] = (GetIOPt(i)+ 48);
			}
			telnet_buffer[16] = '\0';
			telnet_buffer1[16]= '\0';
			if (!(strncmp(telnet_buffer1, telnet_buffer, 16) == 0) && (strlen(telnet_buffer) > 0))
			{
				strcpy(telnet_buffer, telnet_buffer1);
				x = 1;
				return;
			}
			tcp_tick(&s[1]);              /* Need to poll socket 1 for new data */
			tcp_tick(&s[2]);              /* Need to poll socket 2 for new data */
			key_status = inportb(Button_Mem_Location);
			if ( !(key_status == 0) )  						/* A button is pressed */
			{
				status1 = 0;
				tcp_tick(&s[1]);
				return;
			}
			tcp_tick(&s[2]);              /* Need to poll socket 2 for new data */
			temp_rbuffer[0] = '\0';
			if (sock_gets(&s[2],(unsigned char *) temp_rbuffer, 17 ))
			{
				return;
			}
		} while(!(sock_dataready(&s[1])));
		sock_gets(&s[1],(unsigned char *) buffer, 17 );     /* Gets Enter from Telnet screen */
		buffer[0] = '\0';
	}
 sock_err:
	switch (status1)
	{
		case 1 : /* foreign host closed */
			puts("User closed session\n");
			elapsetime = NULL;
			time(&currenttime1);
			return;
		case -1: /* timeout */
			printf("\n\rConnection timed out!\n");
			elapsetime = NULL;
			time(&currenttime1);
			return;
	}
}
/*******************   MAIN   ************************************************/

/*****************************************************************************/
void main(void)
{
	InitIO();				/* Initialize EPX I/O; invoke driver */
	SetAD( 0 );          /* Init input A/D range */
								/*	0 = standard TTL [0 to +5] volts (default)*/
								/*	1 = [  0 to +10] volts */
								/*	2 = [ -5 to + 5] volts */
								/*	3 = [-10 to +10] volts */

	lcd = stdprn;        /* Use stdprn to control LCD*/

	setbuf(lcd, NULL);	/* Disable buffering */

	fprintf(lcd,"%c%c",LCD_CMD,40);        /* With decimal commands */
	fprintf(lcd,"%c%c",LCD_CMD,0xC);       /* With hex commands */
	fprintf(lcd,"%c%c",LCD_CMD,LCD_CMD2);	/* With defined commands */
	fprintf(lcd,"%c%c",LCD_CMD,CLR_HOME);


	LCD_DefineChar (0, LCD_CHAR_PLUS);       // load user-defined characters into LCD
	LCD_DefineChar (1, LCD_CHAR_MINUS);     // load user-defined characters into LCD

	sock_init();
	Intro();
	Mmenu(level, arrow_position, Max_Limit);
}


/***************************************************************************/

/**************************************************************************/
void DispClock()
{
	int pm_time, est = 0;
	/*******   Display clock   ********/
	while(1)
	{
		tcp_tick(&s[1]);  tcp_tick(&s[2]);
		if( sock_established( &s[1] ) || sock_established( &s[2] ) )
		{
			est = 1;
			break;
		}
		key_status = inportb(Button_Mem_Location);
		if( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed*/
			break;

		time(&sec_now);  /* Get current time (in seconds)*/
		strncpy( &t[3], ctime(&sec_now)+11, 8 );
		if( AP_24 == 0 ) // User wants to see AM/PM
		{
			if( (t[3] == '0') || (t[3] == '1' && t[4] == '0') ||
										(t[3] == '1' && t[4] == '1'))// ||
										//(t[3] == '1' && t[4] == '2')   )
			{
				t[13] = 'A';  	/* AM */
			}
			else
			{
				if (t[3] == '1' && t[4] == '2')
				{
				 t[13] = 'P';   /* PM */
				}
				else
				{
					t[13] = 'P';   /* PM */
					pm_time = ( (t[3] - 48)*10 + (t[4] - 48) ) - 12;
					t[3] = (pm_time / 10) + 48;
					t[4] = (pm_time % 10) + 48;
				}
			}
			t[14] = 'M';
		}
		strncpy( &d[3], ctime(&sec_now), 10);
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, "%s\n", t );
		fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
		fprintf( lcd, "%s\n", d );
	}
	if( est == 0)
	{
		level = 0;
	}
	LCDRefresh();
}
/**************************************************************************/

/**************************************************************************/
void InputRelays()
{
	/**** Display input relays values ****/
	char temp_string[17] = { "                \0" };
	while(1)
	{
		fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
		fprintf( lcd, temp_string );

		for(int i=0; i<16; i++)
		{
			temp_string[i] = (GetIOPt(i)+ 48);
		}

		key_status = inportb(Button_Mem_Location);

		if( !( (key_status & PLUS_Mask)==0 ) )  /* PLUS button is pressed*/
		{
			level = 0;
			LCDRefresh();
			break;
		}
		else if( !( (key_status & LEFT_Mask)==0 ) )  /* PLUS button is pressed*/
		{
			level = 0;
			LCDRefresh();
			break;
		}
	}
}
/**************************************************************************/

/**************************************************************************/
void Intro( void )
{
	/* Displays Introduction */
	while( inport(Button_Mem_Location) == 0 )
	{
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, " JKmicrosystems \n" );
		fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
		fprintf( lcd, " LogicFlex-EPX %c\n", 0x00 );
	}
	key_status = inportb(Button_Mem_Location);
	while( !( (key_status & PLUS_Mask)==0 ) )     /*** '+' hit: exit connection... ***/
	{
		break;
	}
}
/*************************************************************************/

/*************************************************************************/
void Mmenu(int &level, int arrow_position[], int Max_Limit[])
{
	int up = 0;
	int quit = 0;
	time_t currenttime2;

	time(&currenttime1);

	fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
	fprintf( lcd, M[0].Option );
	fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
	fprintf( lcd, M[1].Option );

	do
	{
		if ( !(time(&currenttime2) == currenttime1) )
		{
			elapsetime = currenttime2 - currenttime1;

			if ( elapsetime == 600 )         /* 600 sec. = ten min. of no button pressing */
			{
				DispClock();
				elapsetime = NULL;
				time(&currenttime1);
			}
		}
		if ( status1 == 0 )
		{
			telnets(TELNET_PORT);
			tcp_tick(&s[1]);
			status1 = 0;
		}
		key_status = inportb(Button_Mem_Location);
		/* All buttons are zero if they are not pressed */

		/**** UP button is pressed ****/
		if( !( (key_status & UP_Mask)==0 ) )
		{
			DebounceUP();				/* Debouncing for buttons */
			if( level == 0 )
			{
				if( arrow_position[level] > 0 )
				{
					/* Clear current arrow */
					M[ arrow_position[level] ].Option[0] = ' ';
					M[ arrow_position[level] ].Option[1] = ' ';
					/* Move arrow */
					arrow_position[level]--;
					M[ arrow_position[level] ].Option[0] = '-';
					M[ arrow_position[level] ].Option[1] = '>';

					LCDRefresh();
				}
			}
			else if( level == 1 )
			{
				up=1;
				SubMenSel (level, arrow_position, Max_Limit, up);
			}
			else if( level == 2 )
			{
				up=1;
				SubMen12 (level, arrow_position, up);
			}

		}
		/**** DOWN button is pressed ****/
		else if( !( (key_status & DOWN_Mask)==0 ) )
		{
			DebounceDOWN();             /* Debouncing for buttons */
			if( level == 0 )
			{
				if( arrow_position[level] < Max_Limit[level] )
				{
					/* Clear current arrow */
					M[ arrow_position[level] ].Option[0] = ' ';
					M[ arrow_position[level] ].Option[1] = ' ';
					/* Move arrow */
					arrow_position[level]++;
					M[ arrow_position[level] ].Option[0] = '-';
					M[ arrow_position[level] ].Option[1] = '>';

					LCDRefresh();
				}
			}
			else if( level == 1 )
			{
				up=0;
				SubMenSel (level, arrow_position, Max_Limit, up);
			}
			else if( level == 2 )
			{
				up=0;
				SubMen12 (level, arrow_position, up);
			}
		}
		/**** LEFT button is pressed ****/
		else if( !( (key_status & LEFT_Mask)==0 ) )
		{
			DebounceLEFT();           /* Debouncing for buttons */
			if( level == 2 )
			{
			/* Do nothing */
			}
			else if( level > 0 )
			{
				/* For level = 1 only */
				if( level == 1  && arrow_position[0] != 2 && arrow_position[0] != 6 )
				{
					/* Move arrow back to 0 slot */
					(M[ arrow_position[0] ].sub)->SubOption[0][0] = '-';
					(M[ arrow_position[0] ].sub)->SubOption[0][1] = '>';
					(M[ arrow_position[0] ].sub)->SubOption[1][0] = ' ';
					(M[ arrow_position[0] ].sub)->SubOption[1][1] = ' ';

					arrow_position[level] = 0;   /* Reset arrow position */
				}
				level--;
				LCDRefresh();
			}
		}
		/**** RIGHT button is pressed ****/
		/**** OR, special exe_this == 1 ****/
		else if( (!( (key_status & RIGHT_Mask)==0 )) || (exe_this == 1) )
		{
			DebounceRIGHT();           /* Debouncing for buttons */
			if( level == 0 )
			{
				level++;
				LCDRefresh();
			}
			else if( arrow_position[0] == 0 && level == 1 )
			{
				ClintServ (arrow_position);
			}
			else if( arrow_position[0] == 1 && level == 1 )
			{
				SetIP (arrow_position);
			}
			else if( arrow_position[0] == 2 && level == 1 )
			{
				periodic_TCP = ConnWWW ();
			}
			else if( arrow_position[0] == 3 && level == 1 )
			{
				RelayCont ();
			}
			else if( arrow_position[0] == 5 && level == 1 )
			{
				TimeSet ();
			}
			else if( arrow_position[0] == 7 && level == 1 )
			{
				quit = RetOS (quit);
			}
			else if (level == 2)
			{
				WWWConnCont ();
			}
			key_status = 0;
		}
		/**** MINUS button is pressed ****/
		else if( !( (key_status & MINUS_Mask)==0 ) )
		{
			// Clear LCD
			fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
			fprintf( lcd, "                \n" );
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, "                \n" );

			/* Return to DOS */
			exit(-1);
		}
		else if( periodic_TCP == 1 )
		{
			tcp_tick(&s[2]);
		}
	}
	while(quit == 0);
}
/*****************************************************************************/


/****************************************************************************/
int SubMenSel (int level, int arrow_position[], int Max_Limit[], int up)
{
	/* Submenu */

	/* Level 1's SubMENU 0, 1, 3, 4, 5 need to move arrow */
	if (!(arrow_position[0] == 2) || !(arrow_position[0] == 6) )
	{
		/* Move arrow up*/
		if( arrow_position[level] > 0 && up == 1)
		{
			/* Clear current arrow */
			(M[ arrow_position[level-1] ].sub)->SubOption[ arrow_position[level] ][0] = ' ';
			(M[ arrow_position[level-1] ].sub)->SubOption[ arrow_position[level] ][1] = ' ';
			/* Move arrow */
			arrow_position[level]--;
			(M[ arrow_position[level-1] ].sub)->SubOption[ arrow_position[level] ][0] = '-';
			(M[ arrow_position[level-1] ].sub)->SubOption[ arrow_position[level] ][1] = '>';

			LCDRefresh();
		}
		/* Move arrow down*/
		if( arrow_position[level] < Max_Limit[level] && up == 0)
		{
			/* Clear current arrow */
			(M[ arrow_position[level-1] ].sub)->SubOption[ arrow_position[level] ][0] = ' ';
			(M[ arrow_position[level-1] ].sub)->SubOption[ arrow_position[level] ][1] = ' ';
			/* Move arrow */
			arrow_position[level]++;
			(M[ arrow_position[level-1] ].sub)->SubOption[ arrow_position[level] ][0] = '-';
			(M[ arrow_position[level-1] ].sub)->SubOption[ arrow_position[level] ][1] = '>';

			LCDRefresh();
		}
	}
	return level;
}
/*****************************************************************************/


/****************************************************************************/
int SubMen12 (int level, int arrow_position[], int up)
{
	if( level == 2 )
	{
		/* Move arrow up*/
		if( arrow_position[level] > 0 && up == 1)
		{
			/* Clear current arrow */
			Level2Menu[ arrow_position[level] ][0] = ' ';
			Level2Menu[ arrow_position[level] ][1] = ' ';
			/* Move arrow */
			arrow_position[level]--;
			Level2Menu[ arrow_position[level] ][0] = '-';
			Level2Menu[ arrow_position[level] ][1] = '>';

			LCDRefresh();
		}
		/* Move arrow down*/
		if( arrow_position[level] < Max_Limit[level] && up == 0)
		{
			/* Clear current arrow */
			Level2Menu[ arrow_position[level] ][0] = ' ';
			Level2Menu[ arrow_position[level] ][1] = ' ';
			/* Move arrow */
			arrow_position[level]++;
			Level2Menu[ arrow_position[level] ][0] = '-';
			Level2Menu[ arrow_position[level] ][1] = '>';

			LCDRefresh();
		}
	}
	return level;
}
/****************************************************************************/

/****************************************************************************/
void ClintServ (int arrow_position[])
{
	/**** Client/Server Choice ****/

	if( arrow_position[1] == 0 )
	{
		MSmode = 0;               					 /* User selected Client mode */
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, "EPX set to mode:\n" );
		fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
		fprintf( lcd, "<-        Client\n" );

		/* Look at status of LEFT button */
		while( 1 )
		{
			key_status = inportb(Button_Mem_Location);
			/* All buttons are zero by default */
			if( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed so exit */
			{
				DebounceLEFT();        /* Debouncing for buttons */
				break;
			}
		}
	}
	else
	{
		MSmode = 1;          						 /* User selected Server mode */
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, "EPX set to mode:\n" );
		fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
		fprintf( lcd, "<-        Server\n" );

		/* Look at status of LEFT button */
		while( 1 )
		{
			key_status = inportb(Button_Mem_Location);
			/* All buttons are zero by default */
			if( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed so exit */
			{
				DebounceLEFT();        /* Debouncing for buttons */
				break;
			}
		}
	}
	LCDRefresh();
}

/****************************************************************************/

/****************************************************************************/
void SetIP (int arrow_position[])
{
	/**** Set IP address... ****/

	char buffer2[17];
	int i=-1;
	int h=0;

	FILE *fptr;

	if( arrow_position[1] == 0 )
	{
		/* User wants to set EPX's own IP... */

		fprintf(lcd, "%c%c", LCD_CMD, 0xC0 );
		fprintf(lcd, "               %c\0", 0x00 );

		sock_init();   /* Start WATTCP */
		strcpy( own_IP, inet_ntoa( buffer2, gethostid()));
		sock_close(&s);  /* Close WATTCP */

		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, "Enter your IP:  \n" );

		while(continue_flag)
		{
			key_status = inportb(Button_Mem_Location);
			/* All buttons are zero when not pressed */

			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, own_IP );                 /* Prints current IP to lcd */
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, flash_string1 );
			fprintf( lcd, "%c%c", LCD_CMD, 0xCF );  /* Re-position cursor */
			fprintf( lcd, "%c", 0x00 );

			/**** UP button is pressed ****/
			if( !( (key_status & UP_Mask)==0 ) )
			{
				DebounceUP();                /* Debouncing for buttons */
				i++;
				if( i >= 11 )
					i = 0;
				own_IP[h] = select_string[i];
			}
			/**** DOWN button is pressed ****/
			else if( !( (key_status & DOWN_Mask)==0 ) )
			{
				DebounceDOWN();          /* Debouncing for buttons */
				i--;
				if( i < 0 )
					i = 10;
				own_IP[h] = select_string[i];
			}
			/**** LEFT button is pressed ****/
			else if( !( (key_status & LEFT_Mask)==0 ) )
			{
				DebounceLEFT();        /* Debouncing for buttons */
				h--;
				if( h < 0 )
					break;
				else
				{
					strcpy(flash_string1, own_IP);
					flash_string1[h] = '_';
				}
			}
			/**** RIGHT button is pressed ****/
			else if( !( (key_status & RIGHT_Mask)==0 ) )
			{
				DebounceRIGHT();        /* Debouncing for buttons */
				if( h < 14 )  /* Dont serve 14, 15th index */
				{
					h++;
					strcpy(flash_string1, own_IP);
					if( own_IP[h] == ' ' )
						flash_string1[h] = '_';
					else
						flash_string1[h] = ' ';
				 }
			}
			/**** PLUS button is pressed ****/
			else if( !( (key_status & PLUS_Mask)==0 ) )
			{
				if ((fptr = fopen("WATTCP.CFG", "wt")) == NULL)
				{
					printf("Can't open WATTCP.CFG\n");
					puts("");
					exit (1);
				}

				fwrite( "my_ip=", sizeof(char), 6, fptr );
				fwrite( own_IP, sizeof(char), 15, fptr );
				fwrite( "\n", sizeof(char), 1, fptr );
				fwrite( "netmask=255.255.255.0\n", sizeof(char), 22, fptr );
				fwrite( "TFTPD_ONEFILE=YES", sizeof(char), 17, fptr );

				fclose(fptr);

				fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* position cursor */
				fprintf( lcd, "   This EPX's IP\n" );
				fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* re-position cursor */
				fprintf( lcd, "<- is updated   \n" );
				/* Look for status of LEFT button */
				while( 1 )
				{
					key_status = inportb(Button_Mem_Location);
					/* All buttons are zero by default */
					if( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed */
					{
						DebounceLEFT();       /* Debouncing for buttons */
						continue_flag = 0;
						break;
					}
				}
			}
		}
		continue_flag = 1;
	}
	else
	{
		/* User wants to set Remote IP */
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, "Destination IP: \n" );

		while(continue_flag)
		{
			key_status = inportb(Button_Mem_Location);
			/* All buttons are zero when not pressed */
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, remote_IP );
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, flash_string2 );
			fprintf( lcd, "%c%c", LCD_CMD, 0xCF );  /* Re-position cursor */
			fprintf( lcd, "%c", 0x00 );

			/**** UP button is pressed ****/
			if( !( (key_status & UP_Mask)==0 ) )
			{
				DebounceUP();                  /* Debouncing for buttons */
				i++;
				if( i >= 11 )
					i = 0;
				remote_IP[h] = select_string[i];
			 }
			 /**** DOWN button is pressed ****/
			 else if( !( (key_status & DOWN_Mask)==0 ) )
			 {
				DebounceDOWN();          /* Debouncing for buttons */
				i--;
				if( i < 0 )
					i = 10;
				remote_IP[h] = select_string[i];
			 }
			 /**** LEFT button is pressed ****/
			 else if( !( (key_status & LEFT_Mask)==0 ) )
			 {
				DebounceLEFT();               /* Debouncing for buttons */
				h--;
				if( h < 0 )
					break;
				else
				{
					strcpy(flash_string2, remote_IP);
					flash_string2[h] = '_';
				}
			 }
			 /**** RIGHT button is pressed ****/
			 else if( !( (key_status & RIGHT_Mask)==0 ) )
			 {
				DebounceRIGHT/* Debouncing for buttons */();
				if( h < 14 )  /* Dont serve 14th; can't access 15th index */
				{
					h++;
					strcpy(flash_string2, remote_IP);
					if( remote_IP[h] == ' ' )
						flash_string2[h] = '_';
					else
						flash_string2[h] = ' ';
				}
			 }
			 /**** PLUS button is pressed ****/
			 else if( !( (key_status & PLUS_Mask)==0 ) )
			 {
				if( remote_IP[15] == '+' )
					remote_IP[15] = ' ';

				fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
				fprintf( lcd, "RemoteIP updated\n" );
				fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
				fprintf( lcd, "<-              \n" );
				/* Look for status of LEFT button */
				while( 1 )
				{
					key_status = inportb(Button_Mem_Location);
					/* All buttons are zero by default */
					if( !( (key_status & LEFT_Mask)==0 ) )  // LEFT button is pressed */
					{
						DebounceLEFT();         /* Debouncing for buttons */
						continue_flag = 0;
						break;
					}
				}
			}
		}
		continue_flag = 1;
	}
	LCDRefresh();
}
/*****************************************************************************/

/*****************************************************************************/
int ConnWWW ( void )
{
	/**** Connect to WWW... ****/
	fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
	fprintf( lcd, "EPX~~~~~~    EPX\n" );

	sock_init();		/* Start WATTCP */
	tcp_tick( &s[2] );
	/**** Client mode ****/
	if( MSmode == 0 )
	{
		/* Open socket ACTIVE */
		/* EXAMPLE: tcp_open( &s, 0, resolve("10.10.2.68"), TELNET_PORT, NULL ); */
		tcp_open( &s[2], 0, resolve(remote_IP), TELNET_PORT, NULL );
		sock_mode( &s[2], TCP_MODE_ASCII );
		/* Please wait while connection is established... */
		while(1)
		{
			key_status = inportb(Button_Mem_Location);
			tcp_tick( &s[2] );
			if( sock_established( &s[2] ) )
			{
				m_establish = 1;
				break;
			}
			else if( !( (key_status & PLUS_Mask)==0 ) )
			/*** '+' hit: exit connection... ***/
			{
				m_establish = 0;
				exe_this = 0;
				level = 0;
				LCDRefresh();
				break;
			}
			LCDRefresh();
		}
		if( m_establish )
		{
			/* Connection established... */
			fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
			fprintf( lcd, "Connected.    ->\n" );
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, "EPX~~~~~~~~~~EPX\n" );
			/* Look for status of RIGHT button */
			while( 1 )
			{
				key_status = inportb(Button_Mem_Location);
				/* All buttons are zero by default */
				if( !( (key_status & RIGHT_Mask)==0 ) )  // RIGHT button is pressed */
				{
					DebounceLEFT();
					break;
				}
			}
			exe_this = 0;
			/* Display options... */
			level = 2;
			LCDRefresh();
			periodic_TCP = 1;
			m_establish = 0;
		}
	}
	/**** Server mode ****/
	if ( MSmode == 1 )
	{
		/* Open socket PASSIVELY */
		tcp_listen( &s[2], TELNET_PORT, 0, 0, NULL, 0 );
		sock_mode( &s[2], TCP_MODE_ASCII );
		tcp_tick( &s[2] );
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, "Waiting 4 Client\n" );
		fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
		fprintf( lcd, "EPX~~~~~~~~EPX %c\n", 0x00 );

		while(1)
		{
			key_status = inportb(Button_Mem_Location);
			tcp_tick( &s[2] );
			if( sock_established( &s[2] ) )
			{
				s_establish = 1;
				break;
			}
			else if( !( (key_status & PLUS_Mask)==0 ) )
			/*** '+' hit: exit connection... ***/
			{
				fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* position cursor */
				fprintf( lcd, "   Connection  \n" );
				fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* re-position cursor */
				fprintf( lcd, "<- is closed   \n" );
				/* Look for status of LEFT button */
				while( 1 )
				{
					key_status = inportb(Button_Mem_Location);
					/* All buttons are zero by default */

					if( !( (key_status & LEFT_Mask)==0 ) )  // LEFT button is pressed */
					{
						DebounceLEFT();         /* Debouncing for buttons */
						break;
					}
				}
				s_establish = 0;
				exe_this = 0;
				level = 0;
				LCDRefresh();
				break;
			}
		}
		if( s_establish )
		{
			fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
			fprintf( lcd, "(Serving Client)\n" );
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, "EPX~~~~~~~~~~EPX\n" );
			while( tcp_tick(&s[2]) )
			{
				if ( status1 == 0 )               /* If telnet is active */
				{
					telnets(TELNET_PORT);
					tcp_tick(&s[2]);
					status1 = 0;
				}
				for(int i=0; i<16; i++)
				{
					tbuffer[i] = (GetIOPt(i)+ 48);
				}
				tcp_tick(&s[2]);
				tcp_tick(&s[2]);
				if( sock_dataready( &s[2] ) || (!(strcmp(temp_tbuffer, tbuffer) == 0)) )    /* If there is data from Client or the input values change */
				{
					if ( !(strlen(temp_rbuffer) > 0) &&  !(strlen(rbuffer) > 0))
					{
							sock_gets( &s[2], (unsigned char *)rbuffer, 17 );    /* Get message -- when not connected to Telnet */
					}
					else
					{                                       /* Message was already recieved in telnets */
						strncpy(rbuffer, temp_rbuffer, 17);
					}
					fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
					fprintf( lcd, "%s", rbuffer );
					if( (strncmp(rbuffer, "Monitor", 7) == 0) || (!(strncmp(temp_tbuffer, tbuffer, 16) == 0)) )
					{
						tbuffer[16] = '\0';
						strcpy(temp_tbuffer, tbuffer);
						sock_puts(&s[2], (unsigned char *)tbuffer);     /* Send Server's inputs to Client */
					}
					else if( (rbuffer[0]  == '0'  || rbuffer[0]  == '1')
							&& (rbuffer[1]  == '0'  || rbuffer[1]  == '1')
							&& (rbuffer[2]  == '0'  || rbuffer[2]  == '1')
							&& (rbuffer[3]  == '0'  || rbuffer[3]  == '1')
							&& (rbuffer[4]  == '0'  || rbuffer[4]  == '1')
							&& (rbuffer[5]  == '0'  || rbuffer[5]  == '1')
							&& (rbuffer[6]  == '0'  || rbuffer[6]  == '1')
							&& (rbuffer[7]  == '0'  || rbuffer[7]  == '1')
							&& (rbuffer[8]  == '0'  || rbuffer[8]  == '1')
							&& (rbuffer[9]  == '0'  || rbuffer[9]  == '1')
							&& (rbuffer[10] == '0'  || rbuffer[10] == '1')
							&& (rbuffer[11] == '0'  || rbuffer[11] == '1')
							&& (rbuffer[12] == '0'  || rbuffer[12] == '1')
							&& (rbuffer[13] == '0'  || rbuffer[13] == '1')
							&& (rbuffer[14] == '0'  || rbuffer[14] == '1')
							&& (rbuffer[15] == '0'  || rbuffer[15] == '1') )
					{
						for(int i=0; i<16; i++)
						{
							set_port_string[i] = rbuffer[i];
							if( rbuffer[i] == '0' )
								ClrIOPt(i);
							else
								SetIOPt(i);
						}
					}
				}
				key_status = inportb(Button_Mem_Location);
				if( !( (key_status & MINUS_Mask)==0 ) )
				/***   MINUS button is pressed   ***/
				{
					sock_close(&s[2]);
					while( tcp_tick(&s[2]) )
					{ }
					/* Socket completely closed */
					periodic_TCP = 0;
				}
			} /* End while */
			exe_this = 0;
			periodic_TCP = 0;
			s_establish = 0;
			level = 0;
			LCDRefresh();
		}
	}
	return periodic_TCP;
}
/******************************************************************************/

/******************************************************************************/
void RelayCont ( void )
{
	/**** Relay control... ****/
	if( arrow_position[1] == 0 )
	/*** User wants to set output relay ***/
	{
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, "0123456789ABCDEF\n" );

		int toggle = 1;
		int h = 0;
		while(continue_flag2)
		{
			key_status = inportb(Button_Mem_Location);
			/* All buttons are zero when they are not pressed */

			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, set_port_string );
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, flash_string3 );
			fprintf( lcd, "%c%c", LCD_CMD, 0xCF );  /* Re-position cursor */
			fprintf( lcd, "%c", 0x00 );

			/**** UP or Down button is pressed ****/
			if( !( (key_status & UP_Mask)==0 ) || !( (key_status & DOWN_Mask)==0 ) )
			{
				DebounceUP();
            DebounceDOWN();
				if(toggle)
				{
					set_port_string[h] = '1';
					toggle = 0;
				}
				else
				{
					set_port_string[h] = '0';
					toggle = 1;
				}
			}
			/**** LEFT button is pressed ****/
			else if( !( (key_status & LEFT_Mask)==0 ) )
			{
				DebounceLEFT();               /* Debouncing for buttons */
				h--;
				if( h < 0 )
				{
				/* Do nothing*/
				}
				else
				{
					strcpy(flash_string3, set_port_string);
					flash_string3[h] = '_';
				}
			}
			/**** RIGHT button is pressed ****/
			else if( !( (key_status & RIGHT_Mask)==0 ) )
			{
				DebounceRIGHT();        /* Debouncing for buttons */
				if( h < 15 )  /* Can't access 15th index */
				{
					h++;
					strcpy(flash_string3, set_port_string);
					if( set_port_string[h] == ' ' )
						flash_string3[h] = '_';
					else
						flash_string3[h] = ' ';
				}
			}
			/**** PLUS button is pressed ****/
			else if( !( (key_status & PLUS_Mask)==0 ) )
			{
				if( set_port_string[15] == '+' )
					set_port_string[15] = ' ';

				fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
				fprintf( lcd, "Selected relays \n" );
				fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
				fprintf( lcd, "<-      are set \n" );

				for( int i=0; i<16; i++ )
				{
					if( set_port_string[i] == '1' )
					{
						SetIOPt(i);
						clear_port_string[i] = '1';
					}
				}
				/* Look for status of LEFT button */
				while( 1 )
				{
					key_status = inportb(Button_Mem_Location);
					/* all Buttons are zero by default */
					if( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed */
					{
						DebounceLEFT();         /* Debouncing for buttons */
						continue_flag2 = 0;
						break;
					}
				}
			}
		} /* End while: continue_flag2 */
		continue_flag2 = 1;
	}
	else
	/*** User wants to clear output relay ***/
	{
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, "0123456789ABCDEF\n" );

		int toggle = 1;
		int h = 0;
		while(continue_flag2)
		{
			key_status = inportb(Button_Mem_Location);
			/* All buttons are zero when they are not pressed */

			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, clear_port_string );
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, flash_string4 );
			fprintf( lcd, "%c%c", LCD_CMD, 0xCF );  /* Re-position cursor */
			fprintf( lcd, "%c", 0x00 );

			/**** UP or Down button is pressed ****/
			if( !( (key_status & UP_Mask)==0 ) || !( (key_status & DOWN_Mask)==0 ) )
			{
				DebounceUP();
            DebounceDOWN();
				if(toggle)
				{
					clear_port_string[h] = '0';
					toggle = 0;
				}
				else
				{
					clear_port_string[h] = '1';
					toggle = 1;
				}
			}
			/**** LEFT button is pressed ****/
			else if( !( (key_status & LEFT_Mask)==0 ) )
			{
				DebounceLEFT();               /* Debouncing for buttons */
				h--;
				if( h < 0 )
				{
				/* Do nothing*/
				}
				else
				{
					strcpy(flash_string4, clear_port_string);
					flash_string4[h] = '_';
				}
			}
			/**** RIGHT button is pressed ****/
			else if( !( (key_status & RIGHT_Mask)==0 ) )
			{
				DebounceRIGHT();        /* Debouncing for buttons */
				if( h < 15 )  /* Can't access 15th index */
				{
					h++;
					strcpy(flash_string4, clear_port_string);
					if( clear_port_string[h] == ' ' )
						flash_string4[h] = '_';
					else
						flash_string4[h] = ' ';
				}
			}
			/**** PLUS button is pressed ****/
			else if( !( (key_status & PLUS_Mask)==0 ) )
			{
				if( clear_port_string[15] == '+' )
					clear_port_string[15] = ' ';

				fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
				fprintf( lcd, "Selected relays \n" );
				fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
				fprintf( lcd, "<-  are cleared \n" );

				for( int i=0; i<16; i++ )
				{
					if( clear_port_string[i] == '0' )
					{
						ClrIOPt(i);
						set_port_string[i] = '0';
					}
				}
				/* Look for status of LEFT button */
				while( 1 )
				{
					key_status = inportb(Button_Mem_Location);
					/* All buttons are zero by default */
					if( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed */
					{
						DebounceLEFT();         /* Debouncing for buttons */
						continue_flag2 = 0;
						break;
					}
				}
			}
		} /* End while: continue_flag2 */
		continue_flag2 = 1;
	}
	/* Look for a key before break out */
	LCDRefresh();
}
/******************************************************************************/

/******************************************************************************/
void TimeSet ( void )
{
	/**** Time Setting... ****/

	int i=-1;
	int h=0;

	if( arrow_position[1] == 0 )
	/* User wants to set TIME and DATE........ */
	{
		//mytime[15] = '+';
		strcpy(flash_string7, mytime);
		flash_string7[6] = ' ';
		flash_cursor_pos = 0x80;
		fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
		fprintf( lcd, mytime );
		fprintf( lcd, "%c%c", LCD_CMD, 0x8F );  /* Re-position cursor */
		fprintf( lcd, "%c", 0x00 );
		fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
		fprintf( lcd, mydate );

		while(pull_time_flag)
		{
			key_status = inportb(Button_Mem_Location);
			/* All buttons are zero when they are not pressed */

			if( flash_cursor_pos == 0x80 )
			{
				fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
				fprintf( lcd, mytime );
				fprintf( lcd, "%c%c", LCD_CMD, 0x8F );  /* Re-position cursor */
				fprintf( lcd, "%c", 0x00 );
			}
			else
			{
				fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
				fprintf( lcd, mydate );
			}
			fprintf( lcd, "%c%c", LCD_CMD, flash_cursor_pos );
			fprintf( lcd, flash_string7 );

			/**** UP button is pressed ****/
			if( !( (key_status & UP_Mask)==0 ) )
			{
				DebounceUP();                  /* Debouncing for buttons */
				i++;
				if( (i >= 10) || (h==0 && i>2) || (h==2 && i>5) ||
				(h==4 && i>5) || (h==6 && i>1) || (h==8 && i>3) )
				{
					i = 0;
				}
				else
				{
					TimeSet_DigitsUD(h,i);
				}
			}
			/**** DOWN button is pressed ****/
			else if( !( (key_status & DOWN_Mask)==0 ) )
			{
				DebounceDOWN();          /* Debouncing for buttons */
				i--;
				if( i < 0 )
				{
					if( h == 0 )
						i = 2;
					else if( h == 2 )
						i = 5;
					else if( h == 4 )
						i = 5;
					else if( h == 6 )
						i = 1;
					else if( h == 8 )
						i = 3;
					else
						i = 9;
				}
				else
				{
					TimeSet_DigitsUD(h,i);
				}
			}
			/**** LEFT button is pressed ****/
			else if( !( (key_status & LEFT_Mask)==0 ) )
			{
				DebounceLEFT();               /* Debouncing for buttons */
				h--;
				if( h < 0 )
				{
				/* Do nothing*/
				}
				else
				{
					if (h <= 4 )
					{
						TimeSet_DigitsLRa(h);
					}
					else if( h == 5 )
					{
						flash_cursor_pos = 0x80;
						strcpy(flash_string7, mytime);
						flash_string7[13] = ' ';
						fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );
						fprintf( lcd, mydate );
					}
					else if( h == 6 )
					{
						flash_cursor_pos = 0xC0;
						strcpy(flash_string7, mydate);
						flash_string7[6] = ' ';
					}
					else
					{
						TimeSet_DigitsLRb(h);
					}
				}
			}
			/**** RIGHT button is pressed ****/
			else if( !( (key_status & RIGHT_Mask)==0 ) )
			{
				DebounceRIGHT();        /* Debouncing for buttons */
				if( h < 13 )  /* Dont serve 13 */
				{
					h++;

					if (h <= 4 )
					{
						TimeSet_DigitsLRa(h);
					}
					else if( h == 5 )
					{
						flash_cursor_pos = 0x80;
						strcpy(flash_string7, mytime);
						flash_string7[13] = ' ';
					}
					else if( h == 6 )
					{
						flash_cursor_pos = 0xC0;
						strcpy(flash_string7, mydate);
						flash_string7[6] = ' ';
						fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* position cursor */
						fprintf( lcd, mytime );
						fprintf( lcd, "%c%c", LCD_CMD, 0x8F );  /* Re-position cursor */
						fprintf( lcd, "%c", 0x00 );
					}
					else
					{
						TimeSet_DigitsLRb(h);
					}
				}
			}
			/**** PLUS button is pressed ****/
			else if( !( (key_status & PLUS_Mask)==0 ) )
			{
				/*** Use SYSTEM CALL to update TIME and DATE ***/
				mytime[15] = ' ';
				mytime[16] = '\0';
				mydate[16] = '\0';

				if( !(isdigit(mytime[ 6])) || !(isdigit(mytime[ 7])) ||
					 !(isdigit(mytime[ 9])) || !(isdigit(mytime[10])) ||
					 !(isdigit(mytime[12])) || !(isdigit(mytime[13])) ||
					 !(isdigit(mydate[ 6])) || !(isdigit(mydate[ 7])) ||
					 !(isdigit(mydate[ 9])) || !(isdigit(mydate[10])) ||
					 !(isdigit(mydate[12])) || !(isdigit(mydate[13])) ||
					 !(isdigit(mydate[14])) || !(isdigit(mydate[15])) ||
					 ( ((mytime[6]-48)*10 + (mytime[ 7]-48)) > 24 ) ||
					 ( ((mydate[6]-48)*10 + (mydate[ 7]-48)) > 12 ) ||
					 ( ((mydate[9]-48)*10 + (mydate[10]-48)) > 31 )    )
				{
					/* Time and Date string error... */
					fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
					fprintf( lcd, " Incorrect Data \n" );
					fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
					fprintf( lcd, "<-    try again \n" );
				}
				else
				{
					/* Time and Date strings are safe to update... */
					system( mytime );
					system( mydate );

					fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
					fprintf( lcd, "   Time and Date\n" );
					fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
					fprintf( lcd, "<- are updated  \n" );
				}
				/* Look for status of LEFT button */
				while( 1 )
				{
					key_status = inportb(Button_Mem_Location);
					/* All buttons are zero by default */
					if( !( (key_status & LEFT_Mask)==0 ) )  // LEFT button is pressed */
					{
						DebounceLEFT();         /* Debouncing for buttons */
						pull_time_flag = 0;
						break;
					}
				}
			}
		} /* End while: pull_time_flag */
		pull_time_flag = 1;
	}
	/* User wants to toggle AM/PM  V.S. 24 hr clock  */
	else if( level == 1 && arrow_position[0] == 5 )
	{
		if( arrow_position[1] == 1 )
		{
			fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
			fprintf( lcd, "->    AM/PM     \n");
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, "      24 hr     \n" );
			int up = 0;
			int ap24 = 0;
			/* wait for a button to be pressed*/
			while(1)
			{
				/* Look at status of all buttons */
				key_status = inportb(Button_Mem_Location);
				/* All buttons are zero by default */

				/**** UP button is pressed ****/
				if( !( (key_status & UP_Mask)==0 ) && up == 1 )
				{
					DebounceUP();               /* Debouncing for buttons */
					ap24 = 0;
					up = 0;
					fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
					fprintf( lcd, "->    AM/PM     \n");
					fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
					fprintf( lcd, "      24 hr     \n" );
				}
				/**** DOWN button is pressed ****/
				else if( !( (key_status & DOWN_Mask)==0 ) && up == 0 )
				{
					DebounceDOWN();       /* Debouncing for buttons */
					ap24 = 1;
					up = 1;
					fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
					fprintf( lcd, "      AM/PM     \n");
					fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
					fprintf( lcd, "->    24 hr     \n" );
				}
				else if( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed */
				{
					DebounceLEFT();            /* Debouncing for buttons */
					break;
				}
				else if ( !( (key_status & RIGHT_Mask)==0 ) )
				{
					DebounceRIGHT();     /* Debouncing for buttons */
					if( ap24 == 0 && up == 0)
					{
						fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
						fprintf( lcd, "   Clock Type   \n");
						fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
						fprintf( lcd, "<-   AM/PM      \n" );
					}
					else
					{
						fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
						fprintf( lcd, "   Clock Type   \n" );
						fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
						fprintf( lcd, "<-   24 hr      \n");
					}
					/* Look at status of LEFT button */
					while( 1 )
					{
						key_status = inportb(Button_Mem_Location);
						/* All buttons are zero by default */
						if( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed */
						{
							DebounceLEFT();            /* Debouncing for buttons */
							break;
						}
					}
				}
			}
			AP_24 = ap24;
		}
	}
	level = 0;
	LCDRefresh();
}
/******************************************************************************/

/******************************************************************************/
void TimeSet_DigitsUD(int h, int i)
{
	if( h == 0 )
	{
		mytime[6] = select_string[i];
	}
	else if( h == 1 )
	{
		mytime[7] = select_string[i];
	}
	else if( h == 2 )
	{
		mytime[9] = select_string[i];
	}
	else if( h == 3 )
	{
		mytime[10] = select_string[i];
	}
	else if( h == 4 )
	{
		mytime[12] = select_string[i];
	}
	else if( h == 5 )
	{
		mytime[13] = select_string[i];
	}
	else if( h == 6 )
	{
		mydate[6] = select_string[i];
	}
	else if( h == 7 )
	{
		mydate[7] = select_string[i];
	}
	else if( h == 8 )
	{
		mydate[9] = select_string[i];
	}
	else if( h == 9 )
	{
		mydate[10] = select_string[i];
	}
	else if( h == 10 )
	{
		mydate[12] = select_string[i];
	}
	else if( h == 11 )
	{
		mydate[13] = select_string[i];
	}
	else if( h == 12 )
	{
		mydate[14] = select_string[i];
	}
	else if( h == 13 )
	{
		mydate[15] = select_string[i];
	}
}
/******************************************************************************/


/******************************************************************************/
void TimeSet_DigitsLRa(int h)
{
	if( h == 0 )
	{
		flash_cursor_pos = 0x80;
		strcpy(flash_string7, mytime);
		flash_string7[6] = ' ';
	}
	else if( h == 1 )
	{
		flash_cursor_pos = 0x80;
		strcpy(flash_string7, mytime);
		flash_string7[7] = ' ';
	}
	else if( h == 2 )
	{
		flash_cursor_pos = 0x80;
		strcpy(flash_string7, mytime);
		flash_string7[9] = ' ';
	}
	else if( h == 3 )
	{
		flash_cursor_pos = 0x80;
		strcpy(flash_string7, mytime);
		flash_string7[10] = ' ';
	}
	else if( h == 4 )
	{
		flash_cursor_pos = 0x80;
		strcpy(flash_string7, mytime);
		flash_string7[12] = ' ';
	}
}
/******************************************************************************/

/******************************************************************************/
void TimeSet_DigitsLRb(int h)
{
	if( h == 7 )
	{
		flash_cursor_pos = 0xC0;
		strcpy(flash_string7, mydate);
		flash_string7[7] = ' ';
	}
	else if( h == 8 )
	{
		flash_cursor_pos = 0xC0;
		strcpy(flash_string7, mydate);
		flash_string7[9] = ' ';
	}
	else if( h == 9 )
	{
		flash_cursor_pos = 0xC0;
		strcpy(flash_string7, mydate);
		flash_string7[10] = ' ';
	}
	else if( h == 10 )
	{
		flash_cursor_pos = 0xC0;
		strcpy(flash_string7, mydate);
		flash_string7[12] = ' ';
	}
	else if( h == 11 )
	{
		flash_cursor_pos = 0xC0;
		strcpy(flash_string7, mydate);
		flash_string7[13] = ' ';
	}
	else if( h == 12 )
	{
		flash_cursor_pos = 0xC0;
		strcpy(flash_string7, mydate);
		flash_string7[14] = ' ';
	}
	else if( h == 13 )
	{
		flash_cursor_pos = 0xC0;
		strcpy(flash_string7, mydate);
		flash_string7[15] = ' ';
	}
}
/******************************************************************************/

/******************************************************************************/
int RetOS( int quit )
{
	/* Returns urer to operating system */

	fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
	fprintf( lcd, "%c:Back to JK-EPX\n", 0x00 );
	fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
	fprintf( lcd, "%c:  Exit to OS  \n", 0x01 );

	/* wait for a button to be pressed*/
	while(1)
	{
		/* Look at status of all buttons */
		key_status = inportb(Button_Mem_Location);
		/* All buttons are zero by default */

		if( !( (key_status & PLUS_Mask)==0 ) )  /* PLUS button is pressed*/
		{
			quit = 0;
			level--;
			LCDRefresh();
			return quit;
			/* Go back to main menu */
		}
		if( !( (key_status & MINUS_Mask)==0 ) )  /* MINUS button is pressed*/
		{
			/* Clears the LCD */
			fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
			fprintf( lcd, "                \n" );
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, "                \n" );

			quit = 1;       /* Return to DOS */
			return quit;
		}
	}
}
/**************************************************************************/

/******************************************************************************/
void WWWConnCont( void )
{
	/*Controls I/O's While connected*/
	if( level == 2 )
	{
		/*** Client mode ***/
		tcp_tick(&s[2]);

		if( arrow_position[level] == 0 )
		/* Message Server... */
		{
			int i=-1;
			int h=0;

			fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
			fprintf( lcd, "Message Server:%c\n", 0x00 );

			while(pull_message_continue)
			{
				key_status = inportb(Button_Mem_Location);
				/* All buttons are zero when they are not pressed */

				fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
				fprintf( lcd, message );
				fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
				fprintf( lcd, flash_string5 );

				/**** UP button is pressed ****/
				if( !( (key_status & UP_Mask)==0 ) )
				{
					DebounceUP();               /* Debouncing for buttons */
					i++;
					if( i >= 49 )
						i = 0;
					message[h] = ASCII_string[i];
				}
				/**** DOWN button is pressed ****/
				else if( !( (key_status & DOWN_Mask)==0 ) )
				{
					DebounceDOWN();       /* Debouncing for buttons */
					i--;
					if( i < 0 )
						i = 48;
					message[h] = ASCII_string[i];
				}
				/**** LEFT button is pressed ****/
				else if( !( (key_status & LEFT_Mask)==0 ) )
				{
					DebounceLEFT();            /* Debouncing for buttons */
					h--;
					if( h < 0 )
						break;
					else
					{
						strcpy(flash_string5, message);
						flash_string5[h] = '_';
					}
				}
				/**** RIGHT button is pressed ****/
				else if( !( (key_status & RIGHT_Mask)==0 ) )
				{
					DebounceRIGHT();     /* Debouncing for buttons */
					if( h < 15 )  /* Do not serve 15th index */
					{
						h++;
						strcpy(flash_string5, message);
						if( message[h] == ' ' )
							flash_string5[h] = '_';
						else
							flash_string5[h] = ' ';
					}
				}
				/**** PLUS button is pressed ****/
				else if ( !( (key_status & PLUS_Mask)==0 ) )
				{
					tcp_tick( &s[2] );
					strcpy(tbuffer, message);
					sock_puts(&s[2], (unsigned char *)tbuffer);

					fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
					fprintf( lcd, "    Message Sent\n" );
					fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
					fprintf( lcd, "<-              \n" );

					tcp_tick( &s[2] );

					/* Look for status of LEFT button */
					while( 1 )
					{
						key_status = inportb(Button_Mem_Location);
						/* All buttons are zero by default */
						if( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed */
						{
							DebounceLEFT();      /* Debouncing for buttons */
							pull_message_continue = 0;
							break;
						}
					}
				}
			} /* End while: pull_message_continue */
			pull_message_continue = 1;
			LCDRefresh();
		}
		else if ( arrow_position[level] == 1 )
		/* Monitor Inputs... */
		{
			strcpy(tbuffer, "Monitor Inputs..\0");
			sock_puts(&s[2], (unsigned char *)tbuffer);
			fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
			fprintf( lcd, "<-Server Inputs \n" );
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */

			if ((!(temp_lcd[1] == '1')) || (!(temp_lcd[1] == '0')))
				fprintf( lcd, "                \n" );        /* If coming from Level12Menu clear screen */

			while( tcp_tick(&s[2]) )
			{
				tcp_tick(&s[2]);
				tcp_tick(&s[2]);
				if (sock_dataready(&s[2]))
				{
					tcp_tick(&s[2]);
					tcp_tick(&s[2]);

					if( sock_gets(&s[2], (unsigned char *)rbuffer, 17) )
					{
						fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
						if ( strncmp( temp_lcd, rbuffer, 16) == 0 ) /* Inputs did not change */
							fprintf( lcd, "%s", temp_lcd );         /* Prints old inputs */
						else
						{
							fprintf( lcd, "%s", rbuffer ); /* Prints new inputs */
							sprintf( temp_lcd, rbuffer);   /* Stores Servers inputs untill they change */
						}
						strcpy(tbuffer, "Monitor Inputs..\0");
						sock_puts(&s[2], (unsigned char *)tbuffer);
						if ( status1 == 0 )               /* If telnet is active */
						{
							telnets(TELNET_PORT);
							tcp_tick(&s[2]);
							status1 = 0;
						}
					}
				}
				key_status = inportb(Button_Mem_Location);
				/**** LEFT button is pressed ****/
				if( !( (key_status & LEFT_Mask)==0 ) )
				{
					DebounceLEFT();            /* Debouncing for buttons */
					temp_lcd[0] = '\0';
					break;
				}
			} /* End while */

			LCDRefresh();
		}
		else if ( arrow_position[level] == 2 )
		/* Control Outputs... */
		{
			fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
			fprintf( lcd, "0123456789ABCDEF\n" );

			int toggle = 1;
			int h = 0;
			while(continue_flag3)
			{
				key_status = inportb(Button_Mem_Location);
				/* All buttons are zero when they are not pressed */

				fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
				fprintf( lcd, server_output_string );
				fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
				fprintf( lcd, flash_string6 );


				/**** UP button is pressed ****/
				if( !( (key_status & UP_Mask)==0 ) || !( (key_status & DOWN_Mask)==0 ))
				{
					DebounceUP();               /* Debouncing for buttons */
					DebounceDOWN();
					if(toggle)
					{
						server_output_string[h] = '1';
						toggle = 0;
					}
					else
					{
						server_output_string[h] = '0';
						toggle = 1;
					}
				}
				/**** LEFT button is pressed ****/
				else if( !( (key_status & LEFT_Mask)==0 ) )
				{
					DebounceLEFT();            /* Debouncing for buttons */
					h--;
					if( h < 0 )
						break;
					else
					{
						strcpy(flash_string6, server_output_string);
						flash_string6[h] = ' ';
					}
				}
				/**** RIGHT button is pressed ****/
				else if( !( (key_status & RIGHT_Mask)==0 ) )
				{
					DebounceRIGHT();     /* Debouncing for buttons */
					if( h < 15 )  /* Do not serve 15th index */
					{
						h++;
						strcpy(flash_string6, server_output_string);
						flash_string6[h] = ' ';
					}
				}
				/**** PLUS button is pressed ****/
				else if( !( (key_status & PLUS_Mask)==0 ) )
				{
					tcp_tick( &s[2] );
					strcpy(tbuffer, server_output_string);
					strcpy(temp_rbuffer, server_output_string);
					sock_puts(&s[2], (unsigned char *)tbuffer);

					fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
					fprintf( lcd, " Server's Output\n" );
					fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
					fprintf( lcd, "<-   Sent       \n" );

					tcp_tick( &s[2] );

					/* Look for status of LEFT button */
					while( 1 )
					{
						key_status = inportb(Button_Mem_Location);
						/* All buttons are zero by default */
						if( !( (key_status & LEFT_Mask)==0 ) )  // LEFT button is pressed */
						{
							DebounceLEFT();      /* Debouncing for buttons */
							continue_flag3 = 0;
							break;
						}
					}
				}
			} /* End while: continue_flag3 */
			continue_flag3 = 1;
			LCDRefresh();
		}
		else if ( arrow_position[level] == 3 )
		/* Disconnect... */
		{
			//sock_flushnext( &s );
			strcpy(tbuffer, "Disconnecting...\0");
			sock_puts(&s, (unsigned char *)tbuffer);
			fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
			fprintf( lcd, "Disconnecting...\n" );
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, "..Please wait...\n" );

			tcp_tick( &s[2] );
			sock_close(&s[2]);

			/* Socket completely closed */
			level = 0;
			LCDRefresh();
			periodic_TCP = 0;
		}
	}
	sock_err:
	switch(status)
	{
		case 1: /* Foreign host closed */
			fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
			fprintf( lcd, " Remote host is \n" );
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, "<-       closed \n" );
			/* Pull for LEFT button */
			while( 1 )
			{
				key_status = inportb(Button_Mem_Location);
				/* All buttons are zero by default */
				if( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed */
				{
					DebounceLEFT();            /* Debouncing for buttons */
					break;
				}
			}
			exe_this = 0;
			level = 0;
			LCDRefresh();
			status = 0;
			m_establish = 0;
			s_establish = 0;
			break;
		case -1: /* Time-out */
			fprintf( lcd, "%c%c", LCD_CMD, 0x80 );  /* Position cursor */
			fprintf( lcd, "RemoteConnection\n" );
			fprintf( lcd, "%c%c", LCD_CMD, 0xC0 );  /* Re-position cursor */
			fprintf( lcd, "<-     timed out\n" );
			/* Look for status of LEFT button */
			while( 1 )
			{
				key_status = inportb(Button_Mem_Location);
				/* All buttons are zero by default */
				if( !( (key_status & LEFT_Mask)==0 ) )  /* LEFT button is pressed */
				{
					DebounceLEFT();            /* Debouncing for buttons */
					break;
				}
			}
			exe_this = 0;
			level = 0;
			LCDRefresh();
			status = 0;
			m_establish = 0;
			s_establish = 0;
			break;
	}
}
/******************************************************************************/

/******************************************************************************/
void DebounceUP(void)
{
	while( !( (key_status & UP_Mask)==0 ) )
	{
		key_status = inportb(Button_Mem_Location);
		/* All buttons are zero if they are not pressed */
	}
}
/******************************************************************************/

/******************************************************************************/
void DebounceDOWN(void)
{
	while( !( (key_status & DOWN_Mask)==0 ) )           /* Button Debouncing */
	{
		key_status = inportb(Button_Mem_Location);
		/* All buttons are zero if they are not pressed */
	}
}
/******************************************************************************/

/******************************************************************************/
void DebounceLEFT(void)
{
	while( !( (key_status & LEFT_Mask)==0 ) )
	{
		key_status = inportb(Button_Mem_Location);
		/* All buttons are zero if they are not pressed */
	}
}
/******************************************************************************/

/******************************************************************************/
void DebounceRIGHT(void)
{
	while( !( (key_status & RIGHT_Mask)==0 ) )
	{
		key_status = inportb(Button_Mem_Location);
		/* All buttons are zero if they are not pressed */
	}
}
/******************************************************************************/


